#include "controlWindow.h"
using namespace std;

ControlWindow::ControlWindow(QWidget *parent) : QDialog(parent)
{
    setupUi(this);
    QDoubleValidator *validator = new QDoubleValidator(-1e20,1e20,3,this);

    xMinInput->setValidator(validator);
    xMaxInput->setValidator(validator);
    yMinInput->setValidator(validator);
    yMaxInput->setValidator(validator);

    //internal control dialog connections
    connect(labelOnButton,SIGNAL(clicked()),this,SLOT(labelOn()));
    connect(labelOffButton,SIGNAL(clicked()),this,SLOT(labelOff()));
    connect(allLabelsOnButton, SIGNAL(clicked() ), this, SLOT(allLabelsOn()) );
    connect(allLabelsOffButton, SIGNAL(clicked()), this, SLOT(allLabelsOff()) );

    connect(labelColorComboBox, SIGNAL(activated(int)), this,SLOT(colorChanged(int)));
    connect(deciSpinBox,SIGNAL(valueChanged(int)),this,SLOT(formatChanged(int)));

    connect(setButton,SIGNAL(clicked()),this,SLOT(pushOntoStack()));

    labelList.append(namePlotLabelBox);
    labelList.append(indexPlotLabelBox);
    labelList.append(limitsPlotLabelBox);
    labelList.append(modePlotLabelBox);
    labelList.append(timePlotLabelBox);
    labelList.append(gridPlotLabelBox);
    labelList.append(axesPlotLabelBox);

    myTime =0;
    myDecimals =5;
    connect(decimalSpinbox,SIGNAL(valueChanged(int)),this,SLOT(changeDecimals(int)));
}

int ControlWindow::findSelectedLabel()
{
    int selectedLabelIndex;
    for (int i=0;i<7;i++) {
        if(labelList.at(i)->isChecked()){
            selectedLabelIndex =i;
            break;
        }
    }
    return selectedLabelIndex;
}
void ControlWindow::labelOn()
{
    int i = findSelectedLabel();
    emit labelToggled(i,true);
}
void ControlWindow::labelOff()
{
    int i = findSelectedLabel();
    emit labelToggled(i,false);
}
void ControlWindow::allLabelsOn()
{
    for (int i=0;i<7;i++) emit labelToggled(i,true);
}
void ControlWindow::allLabelsOff()
{
    for (int i=0;i<7;i++) emit labelToggled(i,false);
}
void ControlWindow::pushOntoStack()
{
    double xmin = xMinInput->text().toDouble();
    double xmax = xMaxInput->text().toDouble();
    double ymin = yMinInput->text().toDouble();
    double ymax = yMaxInput->text().toDouble();

    plotLimits p;
    p.xMax = xmax;
    p.xMin = xmin;
    p.yMax = ymax;
    p.yMin = ymin;

    emit pushOntoStackSignal(p);
}

void ControlWindow::colorChanged(int color)
{
    int i = findSelectedLabel();
    emit labelColorChanged(i,color);
}
void ControlWindow::formatChanged(int format)
{
    int i = findSelectedLabel();
    emit labelFormatChanged(i,format);
}

void ControlWindow::updateData(xytVector data)
{
    myData = data;
}
void ControlWindow::updateDataIndex(int timeIndex)
{
    myTime = timeIndex;
}
void ControlWindow::showCurrentData()
{
    QString dataString;
    if (myDecimals<5) dataString = "index \t x-value \t y-value\n";
    else dataString = "index \t x-value \t\t y-value\n";

    for (int i=0;i<myData.X.length();i++) {
        dataString.append(QString("%1\t%2\t%3\n").arg(i+1).arg(myData.X.at(myTime).at(i),0,'E',myDecimals).arg(myData.Y.at(myTime).at(i),0,'E',myDecimals) );
    }

    browseDataWindow->setText(dataString);
}
void ControlWindow::changeDecimals(int d)
{
    myDecimals =d;
    showCurrentData();
}
