#include "include/xy.h"

PXYDVEC make_XYDVEC(int n)
{
#define       TYPE     PXYDVEC
#define       X        p->x
#define       Y        p->y
#define       N        p->n
    TYPE       p;

    if( n > 0 ) {
        if( (p = (TYPE) malloc(sizeof(XYDVEC))) != (TYPE) NULL) {
            N = n;
            if( (X = make_DVEC(n)) == NULL ) {
                printf("make_XYDVEC: x DVEC allocation failure.\n");
                p = NULL;
            }
            if( (Y = make_DVEC(n)) == NULL ) {
                if(X) free(X);
                printf("make_XYDVEC: y DVEC allocation failure.\n");
                p = NULL;
            }
            /*       printf("make_XYDVEC: allocated DVEC of length %d.\n",N);  */
        }
    } else {
        p = NULL;
    }
    return p;
}
#undef N

PXYDVEC free_XYDVEC(PXYDVEC p)
{
#define       FREE_IF_NOT_NULL(a) if( a != NULL ) free(a)
    if( p != (TYPE) NULL ) {
        FREE_IF_NOT_NULL(X);
        FREE_IF_NOT_NULL(Y);
        FREE_IF_NOT_NULL(p);
    }
    return (TYPE) NULL;
#undef        FREE_IF_NOT_NULL
}

PXYDVEC dump_XYDVEC(PXYDVEC p, char *s)
{
    if( p != (TYPE) NULL ) {
        printf("dump_xydvec:: %s.\n",s);
        dvprint(X,N,"x"); dvprint(Y,N,"y");
    }
    return p;
}

PXYDVBIND make_XYDVBIND()
{
    PXYDVBIND   lp;

    lp = (PXYDVBIND) malloc(sizeof(XYDVBIND));
    if( lp ) {
        lp->data  = NULL;
        lp->tdata = NULL;
        lp->prev  = NULL;
        lp->next  = NULL;
    }
    return lp;
}

void free_XYDVBIND(PXYDVBIND p)
{
    if( p != NULL ){
        if( p->data ) {
            free(p->data->x);
            free(p->data->y);
        }
        if( p->tdata ) {
            free((p->tdata)->tag);
        }
        free(p);
    }
}

PXYDVBIND link_PXYDVBIND(PXYDVBIND p1,PXYDVBIND p2)
{
    if( p1 ) {
        p1->next = p2;
    }
    if( p2 ) {
        p2->prev = p1;
    }
    return(p2);
}

PXYDVBIND clone_XYDVLIST(PXYDVBIND p)
{
    PXYDVBIND    pp = NULL, newp;
    for( ; p ; p = p->next ) {
        if( !pp ) {
            pp = link_PXYDVBIND(pp,copy_PXYDVBIND(p));
            newp = pp;
        } else {
            pp = link_PXYDVBIND(pp,copy_PXYDVBIND(p));
        }
    }
    return newp;
}

PXYDVBIND_VECN  make_PXYDVBIND_VECN(int n)
{
    PXYDVBIND_VECN   the = {(PXYDVBIND *) NULL,0};

    if( n > 0 ) {
        the.v = (PXYDVBIND *) malloc(n * sizeof(PXYDVBIND));
        the.n = n;
    }

    return(the);
}

void free_PXYDVBIND_VECN(PXYDVBIND_VECN the)
{
    if( the.v ) free(the.v);
}

void free_XYDVLIST(PXYDVBIND p)
{
    PXYDVBIND     lp;
    int           ltrace = OFF, n = 0;

    for( ; p != NULL; ) {
        lp = p->next;
        free_XYDVBIND(p);
        ++n;
        p = lp;
    }
    if( ltrace ) {
        printf("free_XYDVLIST:: %d XY vectors freed.\n",n);
    }
}

PXYDVBIND double_PXYDVBIND(PXYDVBIND p)
{
    int           n = (p->data)->n;

    if( p ) {
        p->data = free_XYDVEC(p->data);
        p->data = make_XYDVEC(2 * n);
        (p->data)->n = 2 * n;
        if( p->tdata ) {
            free_XYTAGVEC(p->tdata);
            p->tdata = make_XYTAGVEC(2 * n);
        }
    }
    return p;
}

PXYDVBIND expand_PXYDVBIND_nfold(PXYDVBIND p,int n_expand)
{
    int           n = (p->data)->n;

    if( p && (n_expand >= 0) ) {
        p->data = free_XYDVEC(p->data);
        p->data = make_XYDVEC(n_expand * n);
        (p->data)->n = n_expand * n;
        if( p->tdata ) {
            free_XYTAGVEC(p->tdata);
            p->tdata = make_XYTAGVEC(n_expand * n);
        }
    }
    return p;
}

PXYDVBIND copy_PXYDVBIND(PXYDVBIND p)
{
    PXYDVBIND   newp = NULL;
#define      N        (p->data)->n
#define      COPY(a)   newp->a = p->a

    if( p ) {
        newp = make_XYDVBIND();
        COPY(index);
        COPY(time);
        COPY(tag);
        newp->data = make_XYDVEC(N);
        DVcopy(p->data->x,newp->data->x,N);
        DVcopy(p->data->y,newp->data->y,N);
        if( p->tdata ) {
            newp->tdata = make_XYTAGVEC(p->tdata->n);
            IVcopy(p->tdata->tag,newp->tdata->tag,p->tdata->n);
        } else {
            newp->tdata = NULL;
        }
    }
    return(newp);
#undef       N
#undef       COPY
}


PXYDVBIND resize_PXYDVBIND_to_n(PXYDVBIND p,int newn)
{
    PXYDVEC       old_data;
    PXYTAGVEC     old_tdata;
    int           n = (p->data)->n;

    if( p && (newn > 0) ) {
        old_data = p->data;
        p->data = make_XYDVEC(newn);
        (p->data)->n = newn;
        free_XYDVEC(old_data);
        old_tdata = p->tdata ? p->tdata : NULL;
        p->tdata = make_XYTAGVEC(newn);
        (p->tdata)->n = newn;
        if( old_tdata ) free_XYTAGVEC(old_tdata);
    }
    return p;
}


double xmin_PXYDVBIND(PXYDVBIND p)
{
    double  xmin = 0;
    if( p ) {
        xmin = dvmin(p->data->x,p->data->n);
        for( p = p->next; p != NULL; p = p->next ) {
            xmin = dmin(xmin,dvmin(p->data->x,p->data->n));
        }
    }
    return(xmin);
}

double xmax_PXYDVBIND(PXYDVBIND p)
{
    double  xmax = 0;
    if( p ) {
        xmax = dvmax(p->data->x,p->data->n);
        for( p = p->next; p != NULL; p = p->next ) {
            xmax = dmax(xmax,dvmax(p->data->x,p->data->n));
        }
    }
    fprintf(stderr,"xmax_PXYDVBIND: xmax: %g.\n",xmax);
    return(xmax);
}

double ymin_PXYDVBIND(PXYDVBIND p)
{
    double  ymin = 0;
    if( p ) {
        ymin = dvmin(p->data->y,p->data->n);
        for( p = p->next; p != NULL; p = p->next ) {
            ymin = dmin(ymin,dvmin(p->data->y,p->data->n));
        }
    }
    return(ymin);
}

double ymax_PXYDVBIND(PXYDVBIND p)
{
    double  ymax = 0;
    if( p ) {
        ymax = dvmax(p->data->y,p->data->n);
        for( p = p->next; p != NULL; p = p->next ) {
            ymax = dmax(ymax,dvmax(p->data->y,p->data->n));
        }
    }
    return(ymax);
}
