#include "plotWidget.h"

#include <QTimer>
#include <QStylePainter>
#include <QPixmap>
#include <QStyleOptionFocusRect>
#include <QWheelEvent>
#include <QMouseEvent>
#include <QMessageBox>
#include <QImageWriter>
#include <QFileDialog>

using namespace std;

//Plotwidget constructor
PlotWidget::PlotWidget(QWidget *parent, int i,bool applyall) : QGLWidget(parent)
{
    this->setMouseTracking(true); //allows mouse move events to be tracked
    this->setContextMenuPolicy(Qt::NoContextMenu);
    //setup initial properties
    index =i;
    isActive=false;
    isSelected =false;
    isApplyAllActive = applyall;
    debug = false;
    timeIndex=0;
    displayMode = ZOOM; //"Easy Zoom" mode

    lineColor =5; //magenta
    markerColor =3; //yellow
    markerType =0; //circle
    markerSize =3; //in pixels
    lineWidth = 2; //in pixels

    linesShown = true;
    markersShown = true;
    fullScale = false;
    tagByDx = true;
    offset =0;
    Margin = 45;

    //initialize label properties
    for (int i=0;i<7;i++){
        labelsVisible[i] = true;
        labelColors[i] = 7;
        labelFormats[i] = 3;
    }
    labelsVisible[5] = false;
    labelsVisible[6] = false;

    //time label
    labelColors[4] = 3; //yellow
    //mode label
    labelColors[3] = 6; //blue
    //title
    labelColors[0] = 6; //blue

    //setup animation timer
    timer = new QTimer(this);
    timeDirection = 0; //0:forward, 1:backward
    breakStatus = 0; //0:off 1:on
    connect(timer, SIGNAL(timeout()), this, SLOT(animate()));

    setBackgroundRole(QPalette::Shadow);

    //cout << backgroundRole() << endl;
    setAutoFillBackground(true);
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    setFocusPolicy(Qt::StrongFocus);
    rubberBandIsShown = false;

    setupButtons();
    setPlotSettings(PlotSettings());
    setupLabels();
}

//exports data to mpeg video
void PlotWidget::makeMpeg()
{
    if (isActive) {

        QString fname = QFileDialog::getSaveFileName(this, tr("Save as .mpg"),
                            QDir::currentPath(),
                            tr("Movies (*.mpg)"));
        if (fname.contains(".mpg")) fname.remove(".mpg");

        emit statusUpdate("Please wait. The mpeg export may take a while");
        QPainter painter;
        for(int i=0;i<data.T.length();i++) {
            changeTimeIndex(i);
            QPixmap mpegPixmap;

            mpegPixmap = QPixmap(width()-width()%16,height()-height()%16);
            mpegPixmap.fill(this,0,0);

            painter.begin(&mpegPixmap);
            painter.initFrom(this);

            //takes care of background for all os (needed for mac)
            QPalette p;
            p.setColor(QPalette::Shadow, Qt::black);
            this->setPalette(p);

            indexLabel = QString("%1/ %2").arg(timeIndex).arg(data.Y.count()-1);
            timeLabel = QString("T:%1").arg(data.T.at(timeIndex),0,'E',labelFormats[4]);

            if (fullScale) initializePlotBoundaries();

            drawGrid(&painter);
            //paint over active/inactive border
            QRect borderRect = QRect(0,0,width()-1,height()-1);
            painter.setPen(Qt::black);
            painter.drawRect(borderRect);

            drawCurves(&painter);
            drawLabels(&painter);

            QString jpegNum= QString("%1").arg(i);
            if (jpegNum.length() >6) {
                cout << "xvs temp files not designed to register 10^7 frames. \n Mpeg will be clipped at 10^7 frames." <<endl;
                break;
            }
            while (jpegNum.length() < 6) {
                jpegNum.prepend("0");
                //cout << qPrintable(jpegNum) << endl;
            }

            mpegPixmap.save(QString("%1%2.jpg").arg("tempJPG_").arg(jpegNum),"JPG");
            painter.end();
        }//end for loop

        if (fname == "") fname == name;
        int retOne = system( qPrintable("ls *jpg | jpeg2yuv -f 25 -I p -j tempJPG_%06d.jpg | mpeg2enc -o" + QString(" %1.mpg").arg(fname)) );

        //delete all temp jpg files
        int retTwo = system("rm -f tempJPG_*.jpg");

        if (retOne == 0 && retTwo ==0){
            emit statusUpdate(QString("%1.mpg successfully written").arg( fname ) );
        }
        else emit statusUpdate(QString("%1.mpg export failed").arg( fname ) );
    }//end if isActive
}
void PlotWidget::exportToImage()
{
    if (isActive) {

        QString fname = QFileDialog::getSaveFileName(this, tr("Save as .jpg"),
                            QDir::currentPath(),
                            tr("Images (*.jpg *.png)"));
        if (!fname.contains(".jpg") && !fname.contains(".png") ) fname.append(".jpg");

        emit statusUpdate("Please wait. The image export may take a while");
        QPainter painter;
        int start;
        int end;

        if (isApplyAllActive) {
            start = 0;
            end = data.T.length();
        }
        else {
            start = timeIndex;
            end = timeIndex +1;
        }

        for(int i=start;i<end;i++) {
            changeTimeIndex(i);
            QPixmap jpegPixmap;

            jpegPixmap = QPixmap(width()-width()%16,height()-height()%16);
            jpegPixmap.fill(this,0,0);

            painter.begin(&jpegPixmap);
            painter.initFrom(this);

            //takes care of background for all os (needed for mac)
            QPalette p;
            p.setColor(QPalette::Shadow, Qt::black);
            this->setPalette(p);

            indexLabel = QString("%1/ %2").arg(timeIndex).arg(data.Y.count()-1);
            timeLabel = QString("T:%1").arg(data.T.at(timeIndex),0,'E',labelFormats[4]);

            if (fullScale) initializePlotBoundaries();

            drawGrid(&painter);
            //paint over active/inactive border
            QRect borderRect = QRect(0,0,width()-1,height()-1);
            painter.setPen(Qt::black);
            painter.drawRect(borderRect);

            drawCurves(&painter);
            drawLabels(&painter);

            QString jpegNum= QString("%1").arg(i);
            if (jpegNum.length() >6) {
                cout << "xvs temp files not designed to register 10^7 frames. \n Mpeg will be clipped at 10^7 frames." <<endl;
                break;
            }
            while (jpegNum.length() < 6) {
                jpegNum.prepend("0");
                //cout << qPrintable(jpegNum) << endl;
            }
            const char *format;
            format = qPrintable(fname.split(".").last().toUpper());
            QString name = fname.split(".").first();
            jpegPixmap.save(QString(name+"_%1.%2").arg(jpegNum).arg(format),format);
            painter.end();
        }//end for loop

        emit statusUpdate(QString("%1 successfully written").arg( fname ) );
    }//end if isActive
}

QSize PlotWidget::minimumSizeHint() const
{
    return QSize(3* Margin, 2*Margin);
}
QSize PlotWidget::sizeHint() const
{
    return QSize(12*Margin, 8*Margin);
}

//creates labels
void PlotWidget::setupLabels()
{
    titleLabel = name;
    timeLabel ="Time";
    indexLabel = QString("%1/ %2").arg(timeIndex, data.Y.count()-1);
    updateMaxMinLabels();
    displayModeLabel="Easy Zoom";
}

//sets up buttons and connects them to actions
void PlotWidget::setupButtons()
{
    forwardButton = new QToolButton(this);
    forwardButton->setIcon(QIcon(":/images/forward.png"));
    forwardButton->adjustSize();
    connect(forwardButton, SIGNAL(clicked()),this,SLOT(forward()));

    backButton = new QToolButton(this);
    backButton->setIcon(QIcon(":/images/back.png"));
    backButton->adjustSize();
    connect(backButton, SIGNAL(clicked()),this,SLOT(back()));

    zoomInButton = new QToolButton(this);
    zoomInButton->setIcon(QIcon(":/images/zoomin.jpg"));
    zoomInButton->adjustSize();
    connect(zoomInButton, SIGNAL(clicked()),this,SLOT(zoomIn()));

    zoomOutButton = new QToolButton(this);
    zoomOutButton->setIcon(QIcon(":/images/zoomout.jpg"));
    zoomOutButton->adjustSize();
    connect(zoomOutButton, SIGNAL(clicked()),this,SLOT(zoomOut()));
}

/*MMMMMMMMMMMM direct modifier functions for plot display properties MMMMMMMMMMMMMMM*/
void PlotWidget::changeIndex(int j)
{
    index =j;
}
void PlotWidget::setActive()
{
    isActive = true;
}
void PlotWidget::setDeactive()
{
    isActive = false;
}
void PlotWidget::setApplyAll(bool y)
{
    isApplyAllActive = y;
}

//these are only viable under active status
void PlotWidget::changeLabelShown(int label, bool shown)
{
    if (isActive || isApplyAllActive) {
        labelsVisible[label] = shown;
        refreshPixmap();
    }
}
void PlotWidget::changeLabelColor(int label, int color)
{
    if (isActive || isApplyAllActive) {
        labelColors[label] = color;
        refreshPixmap();
    }
}
void PlotWidget::changeLabelFormat(int label, int format)
{
    if (isActive || isApplyAllActive) {
        labelFormats[label] = format;
        updateMaxMinLabels();
        timeLabel = QString("T:%1").arg(data.T.at(timeIndex),0,'E',labelFormats[4]);
        refreshPixmap();
    }
}
void PlotWidget::changeTimeIndex(int j)
{
    if (isActive || isApplyAllActive) {
        timeIndex = j;
        emit needUpdateControlDialog("index"); //updates control dialog
    }
}
void PlotWidget::changeLineColor(int newColor)
{
    if (isActive ||isApplyAllActive) {
        lineColor = newColor;
        refreshPixmap();
    }
}
void PlotWidget::changeLineWidth(int newWidth)
{
    if (isActive || isApplyAllActive) {
        lineWidth = newWidth;
        refreshPixmap();
    }
}
void PlotWidget::changeMarkerColor(int newColor)
{
    if (isActive || isApplyAllActive) {
        markerColor = newColor;
        refreshPixmap();
    }
}
void PlotWidget::changeMarkerSize(int newSize)
{
    if (isActive || isApplyAllActive) {
        markerSize = newSize;
        refreshPixmap();
    }
}
void PlotWidget::changeMarkerType(int newType)
{
    if (isActive || isApplyAllActive) {
        markerType = newType;
        refreshPixmap();
    }
}
void PlotWidget::setMarkerVisible(bool v)
{
    if (isActive || isApplyAllActive) {
        markersShown = v;
        refreshPixmap();
    }
}
void PlotWidget::setLinesVisible(bool v)
{
    if (isActive || isApplyAllActive) {
        linesShown = v;
        refreshPixmap();
    }
}
void PlotWidget::changeName(QString newName)
{
    if (isActive || isApplyAllActive) {
        name = newName;
        titleLabel=name;
        data.name = name;
        emit needUpdateControlDialog("name");
        refreshPixmap();
    }
}
void PlotWidget::toggleFullScale(int b)
{
    if (isActive || isApplyAllActive) {
        if (b==1) fullScale =true;
        if (b==0) fullScale =false;
        initializePlotBoundaries();
        refreshPixmap();
    }
}
//handles combobox output & uses setMarkerVisible and setLinesVisible
void PlotWidget::displayTypeHandler(int dtype)
{
    if (debug) cout << "dtype " <<dtype << endl;
    if (dtype ==0) {
        setMarkerVisible(true);
        setLinesVisible(true);
    }
    if (dtype ==1) {
        setMarkerVisible(true);
        setLinesVisible(false);
    }
    if (dtype ==2) {
        setMarkerVisible(false);
        setLinesVisible(true);
    }
}
/*MMMMMMMMMMMM direct modifier functions for plot display properties MMMMMMMMMMMMMMM*/

/**************mode related functions**************/
//time index modifiers
void PlotWidget::timePlusPlus()
{
    if (isActive || isApplyAllActive) {
        timeIndex++;
        if ((timeIndex==data.Y.count()) && (breakStatus == 1)) {
            stopAnimate();
            timeIndex--;
            return;
        }
        changeTimeIndex(timeIndex%(data.Y.count()) );

        indexLabel = QString("%1/ %2").arg(timeIndex).arg(data.Y.count()-1);
        timeLabel = QString("T:%1").arg(data.T.at(timeIndex),0,'E',labelFormats[4]);

        if (fullScale) initializePlotBoundaries();

        refreshPixmap();
    }
}
void PlotWidget::timeMinusMinus()
{
    if (isActive || isApplyAllActive) {
        if (timeIndex >0) changeTimeIndex(timeIndex-1);

        else{

            changeTimeIndex(data.T.count()-1);
            if (breakStatus == 1)
            {
                stopAnimate();
                changeTimeIndex(0);
            }
        }

        indexLabel = QString("%1/ %2").arg(timeIndex).arg(data.Y.count()-1);
        timeLabel = QString("T:%1").arg(data.T.at(timeIndex),0,'E',labelFormats[4]);

        if (fullScale) initializePlotBoundaries();

        refreshPixmap();
    }
}
void PlotWidget::setTimeIndex(int t)
{
    if (isActive) { //missing the customary || isAllActive b/c datasets w/ diff lengths would create segfaults
        if ((t>=0) && (t<data.Y.count()) )
        {
            changeTimeIndex(t);

            indexLabel = QString("%1/ %2").arg(timeIndex).arg(data.Y.count()-1);
            refreshPixmap();
        }
    }
}

//Animation section
void PlotWidget::startAnimate()
{
    if (isActive || isApplyAllActive) timer->start(60); //timer is connected to animate()
}
void PlotWidget::stopAnimate()
{
    if (isActive || isApplyAllActive) timer->stop();
}
void PlotWidget::animate()
{
    if (timeDirection ==0) timePlusPlus();
    if (timeDirection ==1) timeMinusMinus();
}
void PlotWidget::changeTimeDirection(int direction)
{
    if (isActive || isApplyAllActive) timeDirection = direction;
}
void PlotWidget::changeBreakStatus(int b)
{
    if (isActive || isApplyAllActive) breakStatus =b;
}

void PlotWidget::changeDisplayMode(int dmode)
{
    if (dmode == SELECTION) { //selection mode targets all widgets. Only relevent for selection mode
            displayMode = dmode;
            stopAnimate();
            forwardButton->setShown(false);
            backButton->setShown(false);
            zoomInButton->setShown(false);
            zoomOutButton->setShown(false);
            displayModeLabel="Selection";
        }
    if (dmode == DESELECTION) { //deselection resets all widgets
        displayMode = ZOOM;
        isSelected = false;
        forwardButton->setShown(true);
        backButton->setShown(true);
        zoomInButton->setShown(true);
        zoomOutButton->setShown(true);
        displayModeLabel="Easy Zoom";
        refreshPixmap();
    }
    if (isActive || isApplyAllActive) {
        displayMode = dmode;
        if (dmode == ZOOM) {
            stopAnimate();
            forwardButton->setShown(true);
            backButton->setShown(true);
            zoomInButton->setShown(true);
            zoomOutButton->setShown(true);
            displayModeLabel="Easy Zoom";
            refreshPixmap();
        }
        if (dmode == XZOOM) {
            stopAnimate();
            forwardButton->setShown(false);
            backButton->setShown(false);
            zoomInButton->setShown(false);
            zoomOutButton->setShown(false);
            displayModeLabel="X-zoom";
            refreshPixmap();
        }
        if (dmode == YZOOM) {
            stopAnimate();
            forwardButton->setShown(false);
            backButton->setShown(false);
            zoomInButton->setShown(false);
            zoomOutButton->setShown(false);
            displayModeLabel="Y-zoom";
            refreshPixmap();
        }
        if (dmode == XYZOOM) {
            stopAnimate();
            forwardButton->setShown(false);
            backButton->setShown(false);
            zoomInButton->setShown(false);
            zoomOutButton->setShown(false);
            displayModeLabel="XY-zoom";
            refreshPixmap();
        }
        if (dmode == STEP) {
            stopAnimate();
            forwardButton->setShown(false);
            backButton->setShown(false);
            zoomInButton->setShown(false);
            zoomOutButton->setShown(false);
            displayModeLabel="Step";
            refreshPixmap();
        }
        if (dmode == AUTO) {
            startAnimate();
            forwardButton->setShown(false);
            backButton->setShown(false);
            zoomInButton->setShown(false);
            zoomOutButton->setShown(false);
            displayModeLabel="Auto";
        }
        if (dmode == FREEZE) {
            stopAnimate();
            forwardButton->setShown(false);
            backButton->setShown(false);
            zoomInButton->setShown(false);
            zoomOutButton->setShown(false);
            displayModeLabel="Freeze";
            refreshPixmap();
        }
        emit needUpdateControlDialog("mode");
    }//end if isactive
}
/************end mode related functions************/

//sets plot settings to specified var (called only in contructor)
void PlotWidget::setPlotSettings(const PlotSettings &settings)
{
    zoomStack.clear();
    zoomStack.append(settings);
    curZoom =0;
    forwardButton->hide();
    backButton->hide();
    //refreshPixmap();
}
void PlotWidget::setMinX(QString x)
{
    if (x.toDouble() < zoomStack[curZoom].maxX) {
        zoomStack[curZoom].minX = x.toDouble();
    }
    refreshPixmap();
}
void PlotWidget::setMaxX(QString x)
{
    if (x.toDouble() > zoomStack[curZoom].minX) {
        zoomStack[curZoom].maxX = x.toDouble();
    }
    refreshPixmap();
}
void PlotWidget::setMinY(QString y)
{
    if (y.toDouble() < zoomStack[curZoom].maxY) {
        zoomStack[curZoom].minY = y.toDouble();
    }
    refreshPixmap();
}
void PlotWidget::setMaxY(QString y)
{
    if (y.toDouble() > zoomStack[curZoom].minY) {
        zoomStack[curZoom].maxY = y.toDouble();
    }
    refreshPixmap();
}

/*#######################################################################
##########################Accessors######################################
#######################################################################*/
plotLimits PlotWidget::getPlotLimits()
{
    plotLimits maxMins;
    maxMins.xMin = zoomStack[curZoom].minX;
    maxMins.xMax = zoomStack[curZoom].maxX;
    maxMins.yMin = zoomStack[curZoom].minY;
    maxMins.yMax = zoomStack[curZoom].maxY;

    return maxMins;
}
QMap<int,QVector<double> > PlotWidget::getZoomStack()
{
    QMap<int,QVector<double> > data;
    for (int i=0;i<zoomStack.count();i++) {
        data[i].append(zoomStack.at(i).minX);
        data[i].append(zoomStack.at(i).maxX);
        data[i].append(zoomStack.at(i).minY);
        data[i].append(zoomStack.at(i).maxY);
    }//retrieves zoominfo
    data[zoomStack.count()].append(curZoom);
    return data;
}
int PlotWidget::getIndex()
{
    return index;
}
QString PlotWidget::getName()
{
    return name;
}
int PlotWidget::getLineColor()
{
    return lineColor;
}
int PlotWidget::getMarkerColor()
{
    return markerColor;
}
int PlotWidget::getLineWidth()
{
    return lineWidth;
}
int PlotWidget::getMarkerSize()
{
    return markerSize;
}
int PlotWidget::getMarkerType()
{
    return markerType;
}
int PlotWidget::getTimeDirection()
{
    return timeDirection;
}
int PlotWidget::getBreakStatus()
{
    return breakStatus;
}
int PlotWidget::getTimeIndex()
{
    return timeIndex;
}
int PlotWidget::getMaxTimeIndex()
{
    return data.Y.count()-1;
}
int PlotWidget::getDisplayMode()
{
    return displayMode;
}
bool PlotWidget::getGridShown()
{
    return labelsVisible[5];
}
bool PlotWidget::getMarkersShown()
{
    return markersShown;
}
bool PlotWidget::getLinesShown()
{
    return linesShown;
}
int PlotWidget::getFullScale()
{
    if (fullScale) return 1;
    else return 0;
}
/*#######################################################################
##########################End Accessors######################################
#######################################################################*/


//sets all Y data to cMap
void PlotWidget::setDataVec(xytVector vec)
{
    data = vec;
    timeLabel = QString("T:%1").arg(data.T.at(timeIndex),0,'E',labelFormats[4]);
}
//adds f(x,t) to list of funcs (so far only used with convergence tests)
void PlotWidget::addCurveMap(xytVector vec)
{
    curveMap.append(vec);
    timeLabel = QString("T:%1").arg(data.T.at(timeIndex),0,'E',labelFormats[4]);
}
/*remove specified f(x,@t) & redraws
void PlotWidget::trim(int id)
{
    data.Y.removeAt(id);
    refreshPixmap();
}*/


/*################Plot zoom methods####################*/
//moves one index back in saved plot settings array
void PlotWidget::back()
{
    if (curZoom > 0) {
        --curZoom;
        backButton->setEnabled(curZoom>0);
        if (displayMode == ZOOM) {
            forwardButton->setEnabled(true);
            forwardButton->show();
        }
        updateMaxMinLabels();
        emit needUpdateControlDialog("view"); //updates control dialog
        refreshPixmap();
    }
}
//moves one index forward in saved plot settings array
void PlotWidget::forward()
{
    if (curZoom < zoomStack.count()-1){
        ++curZoom;
        forwardButton->setEnabled(curZoom < zoomStack.count()-1);
        if (displayMode == ZOOM){
            backButton->setEnabled(true);
            backButton->show();
        }

        updateMaxMinLabels();
        emit needUpdateControlDialog("view"); //updates control dialog
        refreshPixmap();
    }
}
//(2x) zoom in
void PlotWidget::zoomIn()
{
    PlotSettings prevSettings = zoomStack[curZoom];
    PlotSettings settings;

    settings.minX = prevSettings.minX + prevSettings.spanX()/4;
    settings.maxX = prevSettings.maxX - prevSettings.spanX()/4;
    settings.minY = prevSettings.minY + prevSettings.spanY()/4;
    settings.maxY = prevSettings.maxY - prevSettings.spanY()/4;
    settings.adjust();

    zoomStack.resize(curZoom +1);
    zoomStack.append(settings);
    forward();
}
void PlotWidget::zoomInX(double x)
{
    PlotSettings prevSettings = zoomStack[curZoom];
    PlotSettings settings;
    settings.minY = prevSettings.minY;
    settings.maxY = prevSettings.maxY;

    QRect rect(Margin, Margin, width()-2*Margin,height()-2*Margin);
    x = (prevSettings.spanX()/rect.width() ) * (x- rect.left()) +prevSettings.minX;

    if (x+prevSettings.spanX()/4. > prevSettings.maxX) {
        settings.maxX = prevSettings.maxX;
        settings.minX = prevSettings.maxX - prevSettings.spanX()/2.;
    }
    else if (x-prevSettings.spanX()/4. < prevSettings.minX) {
        settings.minX = prevSettings.minX;
        settings.maxX = prevSettings.minX + prevSettings.spanX()/2.;
    }
    else {
        settings.minX = x - prevSettings.spanX()/4;
        settings.maxX = x + prevSettings.spanX()/4;
    }

    zoomStack.resize(curZoom +1);
    zoomStack.append(settings);
    forward();
}
void PlotWidget::zoomInY(double y)
{
    PlotSettings prevSettings = zoomStack[curZoom];
    PlotSettings settings;
    settings.minX = prevSettings.minX;
    settings.maxX = prevSettings.maxX;

    QRect rect(Margin, Margin, width()-2*Margin,height()-2*Margin);
    y = (prevSettings.spanY()/rect.height() ) * (rect.bottom() - y) + prevSettings.minY;

    //cout << y << endl;
    if (y+prevSettings.spanY()/4. > prevSettings.maxY) {
        settings.minY = prevSettings.maxY - prevSettings.spanY()/2.;
        settings.maxY = prevSettings.maxY;
    }
    else if (y-prevSettings.spanY()/4. < prevSettings.minY) {
        settings.minY = prevSettings.minY;
        settings.maxY = prevSettings.minY + prevSettings.spanY()/2.;
    }
    else {
        settings.minY = y - prevSettings.spanY()/4;
        settings.maxY = y + prevSettings.spanY()/4;
    }

    zoomStack.resize(curZoom +1);
    zoomStack.append(settings);
    forward();
}
void PlotWidget::zoomInXY(QPointF pos)
{
    PlotSettings prevSettings = zoomStack[curZoom];
    PlotSettings settings;

    QRect rect(Margin, Margin, width()-2*Margin,height()-2*Margin);
    double x = pos.x();
    x = (prevSettings.spanX()/rect.width() ) * (x- rect.left())+prevSettings.minX;
    double y = pos.y();
    y = (prevSettings.spanY()/rect.height() ) * (rect.bottom() - y)+prevSettings.minY;

    if (x+prevSettings.spanX()/4. > prevSettings.maxX) {
        settings.minX = prevSettings.maxX - prevSettings.spanX()/2.;
        settings.maxX = prevSettings.maxX;
    }
    else if (x-prevSettings.spanX()/4. < prevSettings.minX) {
        settings.minX = prevSettings.minX;
        settings.maxX = prevSettings.minX + prevSettings.spanX()/2.;
    }
    else {
        settings.minX = x - prevSettings.spanX()/4;
        settings.maxX = x + prevSettings.spanX()/4;
    }

    if (y+prevSettings.spanY()/4. > prevSettings.maxY) {
        settings.minY = prevSettings.maxY - prevSettings.spanY()/2.;
        settings.maxY = prevSettings.maxY;
    }
    else if (y-prevSettings.spanY()/4. < prevSettings.minY) {
        settings.minY = prevSettings.minY;
        settings.maxY = prevSettings.minY + prevSettings.spanY()/2.;
    }
    else {
        settings.minY = y - prevSettings.spanY()/4;
        settings.maxY = y + prevSettings.spanY()/4;
    }

    zoomStack.resize(curZoom +1);
    zoomStack.append(settings);
    forward();
}
//(2x) zoom out
void PlotWidget::zoomOut()
{
    PlotSettings prevSettings = zoomStack[curZoom];
    PlotSettings settings;

    settings.minX = prevSettings.minX - prevSettings.spanX()/2;
    settings.maxX = prevSettings.maxX + prevSettings.spanX()/2;
    settings.minY = prevSettings.minY - prevSettings.spanY()/2;
    settings.maxY = prevSettings.maxY + prevSettings.spanY()/2;
    settings.adjust();

    zoomStack.resize(curZoom +1);
    zoomStack.append(settings);
    forward();
}
/*################End Plot zoom methods####################*/


/*&&&&&&&&&&&&& Keyboard and mouse event implementations &&&&&&&&&&&&&&&&&*/
//reimplemented to allow rubberband zoom
void PlotWidget::mouseMoveEvent(QMouseEvent *event)
{
    if (labelsVisible[6] || labelsVisible[5]) Margin = 45;
    else Margin = 1;
    QRect rect(Margin, Margin, width()-2*Margin,height()-2*Margin);

    PlotSettings settings = zoomStack[curZoom];
    double dx = event->pos().x() - rect.left();
    double dy = rect.bottom() - event->pos().y();
    double x = settings.minX + (dx* settings.spanX()/ (rect.width()-1) );
    double y = settings.minY + (dy* settings.spanY()/(rect.height()-1) );

    QPointF p;
    p.setX(x);
    p.setY(y);
    emit mouseMoved(p, index);

    if (displayMode ==ZOOM) {
        if (rubberBandIsShown) {
            updateRubberBandRegion();
            rubberBandRect.setBottomRight(event->pos());
            updateRubberBandRegion();
        }
    }

}
//reimplemented to allow rubberband zoom
void PlotWidget::mousePressEvent(QMouseEvent *event)
{
    emit mouseClicked(event->pos());
    switch(displayMode)
    {
    case ZOOM: //zoom mode
        {QRect rect(Margin, Margin, width() -2*Margin,height() -2*Margin);
            if (event->button() == Qt::LeftButton){
                if ( rect.contains(event->pos()) ) { //begins a rubberband zoom action
                    rubberBandIsShown = true;
                    rubberBandRect.setTopLeft(event->pos());
                    rubberBandRect.setBottomRight(event->pos());
                    updateRubberBandRegion();
                    setCursor(Qt::CrossCursor);
                } //end if contains
            }//end if event
            break;}
    case XZOOM:
        if (event->button() == Qt::LeftButton) back();
        if (event->button() == Qt::RightButton) zoomInX(event->posF().x());
        break;
    case YZOOM:
        if (event->button() == Qt::LeftButton) back();
        if (event->button() == Qt::RightButton) zoomInY(event->posF().y());
        break;
    case XYZOOM:
        if (event->button() == Qt::LeftButton) back();
        if (event->button() == Qt::RightButton) zoomInXY(event->posF());
        break;
    case STEP: //step mode
        if (event->button() == Qt::LeftButton) timePlusPlus();
        break;
    case AUTO: //auto mode
        if (event->button() == Qt::LeftButton) startAnimate();
        if (event->button() == Qt::RightButton) stopAnimate();
        break;
    case FREEZE:
        break;
    case SELECTION: //selection mode
        switch (event->button()) {
        case Qt::LeftButton:
            emit widgetSelected(index);
            isSelected = true;
            refreshPixmap();
            break;
        case Qt::RightButton:
            emit widgetDeselected(index);
            isSelected = false;
            refreshPixmap();
            break;
        default:
            QWidget::mousePressEvent(event);
        }//end switch
    }
}
//reimplemented to give buttons const pixel distances from the edge
void PlotWidget::resizeEvent(QResizeEvent * /*event*/)
{
    int x = width()- (forwardButton->width() + backButton->width() +10);
    forwardButton->move(x+ forwardButton->width(),25);
    backButton->move(x, 25);
    zoomInButton->move(x,25+forwardButton->height());
    zoomOutButton->move(x+forwardButton->width(),25+forwardButton->height());
    refreshPixmap();
}
//reimplemented to allow a rubberband zoom on selected area
void PlotWidget::mouseReleaseEvent(QMouseEvent *event)
{
    if (displayMode == ZOOM) {
        if ((event->button() == Qt::LeftButton) && rubberBandIsShown) {
            rubberBandIsShown =false;
            updateRubberBandRegion();
            unsetCursor();

            QRect rect = rubberBandRect.normalized();
            if (rect.width() < 4 || rect.height() < 4) return;
            rect.translate(-Margin,-Margin);

            PlotSettings prevSettings = zoomStack[curZoom];
            PlotSettings settings;
            double dx = prevSettings.spanX() / (width() - 2*Margin);
            double dy = prevSettings.spanY() / (height() - 2*Margin);

            settings.minX = prevSettings.minX + dx*rect.left();
            settings.maxX = prevSettings.minX + dx*rect.right();
            settings.minY = prevSettings.maxY - dy*rect.bottom();
            settings.maxY = prevSettings.maxY - dy*rect.top();
            settings.adjust();

            zoomStack.resize(curZoom +1);
            zoomStack.append(settings);
            forward();
        }
    }
}
//implements all keyboard shortcuts
void PlotWidget::keyPressEvent(QKeyEvent *event)
{
    switch (event->key())
    {
    case Qt::Key_Plus:
        forward();
        break;
    case Qt::Key_Minus:
        back();
        break;
    case Qt::Key_Left:
        if (displayMode ==ZOOM){
            zoomStack[curZoom].scroll(-1,0);
            updateMaxMinLabels();
            emit needUpdateControlDialog("view"); //updates control dialog
            refreshPixmap();
        }
        if (displayMode==STEP) timeMinusMinus();
        break;
    case Qt::Key_Right:
        if (displayMode ==ZOOM) {
            zoomStack[curZoom].scroll(+1,0);
            updateMaxMinLabels();
            emit needUpdateControlDialog("view"); //updates control dialog
            refreshPixmap();
        }
        if (displayMode==STEP) timePlusPlus();
        break;
    case Qt::Key_Down:
        if (displayMode ==ZOOM) {
            zoomStack[curZoom].scroll(0,-1);
            emit needUpdateControlDialog("view"); //updates control dialog
            updateMaxMinLabels();
            refreshPixmap();
        }
        break;
    case Qt::Key_Up:
        if (displayMode ==ZOOM) {
            zoomStack[curZoom].scroll(0,+1);
            emit needUpdateControlDialog("view"); //updates control dialog
            updateMaxMinLabels();
            refreshPixmap();
        }
        break;

    default:
        QWidget::keyPressEvent(event);
    }
}
//gives scroll ability to mouse wheel
void PlotWidget::wheelEvent(QWheelEvent *event)
{
    if (displayMode ==ZOOM) {
        int numDegrees = event->delta() / 8;
        int numTicks = numDegrees/15;

        if (event->orientation() == Qt::Horizontal) {
            zoomStack[curZoom].scroll(numTicks,0);
        } else {
            zoomStack[curZoom].scroll(0,numTicks);
        }

        updateMaxMinLabels();
        emit needUpdateControlDialog("view"); //updates control dialog
        refreshPixmap();
    }
}
/*&&&&&&&&&&&&& End Keyboard and mouse event implementations &&&&&&&&&&&&&&&&&*/


//updates min max labels with suitable number format
void PlotWidget::updateMaxMinLabels()
{
    PlotSettings settings = zoomStack[curZoom];
    QString xMax;
    QString xMin;
    QString yMax;
    QString yMin;

    xMax = QString("%1").arg(settings.maxX,0,'E',labelFormats[2]);
    xMin = QString("%1").arg(settings.minX,0,'E',labelFormats[2]);
    yMax = QString("%1").arg(settings.maxY,0,'E',labelFormats[2]);
    yMin = QString("%1").arg(settings.minY,0,'E',labelFormats[2]);

    //update labels
    lowerLimitLabels = QString("%1, %2").arg(xMin).arg(yMin);
    upperLimitLabels = QString("%1, %2").arg(xMax).arg(yMax);
}

void PlotWidget::updateRubberBandRegion()
{
    QRect rect = rubberBandRect.normalized();
    update(rect.left(), rect.top(),rect.width(),1);
    update(rect.left(), rect.top(),1,rect.height());
    update(rect.left(), rect.bottom(),rect.width(),1);
    update(rect.right(), rect.top(),1,rect.height());
}


/****** Drawing functions ******/
//reimplements paintEvent to allow rubberband zoom
void PlotWidget::paintEvent(QPaintEvent * /*event*/)
{
    QStylePainter painter(this);
    painter.drawPixmap(0,0,pixmap);

    if (rubberBandIsShown){
        painter.setPen(palette().light().color());
        painter.drawRect(rubberBandRect.normalized().adjusted(0,0,-1,-1));

    }

    if (hasFocus()) {
        QStyleOptionFocusRect option;
        option.initFrom(this);
        option.backgroundColor = Qt::black;
        painter.drawPrimitive(QStyle::PE_FrameFocusRect, option);
    }
}
//redraws plot
void PlotWidget::refreshPixmap()
{
    pixmap = QPixmap(size());
    pixmap.fill(this,0,0);

    QPainter painter;
    painter.begin(&pixmap);
    painter.initFrom(this);

    //takes care of background for all os (needed for mac)
    QPalette p;
    p.setColor(QPalette::Shadow, Qt::black);
    this->setPalette(p);

    drawGrid(&painter);
    drawCurves(&painter);
    drawLabels(&painter);
    painter.end();
    update();
}
//draws grid and ticks & tick labels
void PlotWidget::drawGrid(QPainter *painter)
{
    static const QColor colorForIds[8] = {Qt::black,Qt::red,Qt::green, Qt::yellow,
                                          Qt::blue,Qt::magenta,Qt::cyan,Qt::white };

    //determines margins for plot. Removes margins if grid/axes not shown
    if (labelsVisible[6] || labelsVisible[5]) Margin = 45;
    else Margin = 1;

    QRect rect(Margin, Margin, width()-2*Margin, height()-2*Margin);
    if (!rect.isValid()) return;

    PlotSettings settings = zoomStack[curZoom];

    //Debug statement
    if (debug) settings.info();
    if (labelsVisible[6] || labelsVisible[5]) {
        for (int i=0; i<=settings.numXTicks; i++) {
            int x = rect.left() + (i * (rect.width() -1) /settings.numXTicks);
            double label = settings.minX + (i*settings.spanX()/settings.numXTicks);
            QString labelString;

            labelString = QString("%1").arg(label,0,'g',3);

            painter->setPen(colorForIds[labelColors[5]]);
            if (labelsVisible[5]) painter->drawLine(x, rect.top(), x,rect.bottom());
            painter->setPen(colorForIds[labelColors[6]]);
            if (labelsVisible[6]){
                painter->drawLine(x, rect.bottom(), x, rect.bottom()+5);
                painter->drawText(x-50, rect.bottom()+5, 100, 15, Qt::AlignCenter | Qt::AlignTop,
                              labelString);
            }
        }
        for (int j=0;j<=settings.numYTicks;j++) {
            int y = rect.bottom() - (j* (rect.height()-1) / settings.numYTicks);
            double label = settings.minY + (j*settings.spanY()/settings.numYTicks);
            if (abs(label) < 1e-10) label =0;
            QString labelString;

            labelString = QString("%1").arg(label,0,'g',3);

            painter->setPen(colorForIds[labelColors[5]]);
            if (labelsVisible[5]) painter->drawLine(rect.left(),y,rect.right(),y);
            painter->setPen(colorForIds[labelColors[6]]);
            if (labelsVisible[6]) {
                painter->drawLine(rect.left()-5,y,rect.left(),y);
                painter->drawText(rect.left()-Margin,y-10,Margin-5,20,
                              Qt::AlignRight | Qt::AlignVCenter,
                              labelString);
            }
        }
        if (labelsVisible[5] || labelsVisible[6])painter->drawRect(rect.adjusted(0,0,-1,-1));
    }//end axesShown

    //paint active/inactive border
    QRect borderRect = QRect(0,0,width()-1,height()-1);
    if (isActive) painter->setPen(Qt::green);
    else painter->setPen(Qt::red);
    if (isSelected) painter->setPen(Qt::blue);
    painter->drawRect(borderRect);
}
//draws curves
void PlotWidget::drawCurves(QPainter *painter)
{
    //define possible colors
    static const QColor colorForIds[8] =
    {Qt::black,Qt::red,Qt::green, Qt::yellow,
     Qt::blue,Qt::magenta,Qt::cyan,Qt::white };
    //import plot data
    PlotSettings settings = zoomStack[curZoom];

    //set up plot area inside margins
    if (labelsVisible[6] || labelsVisible[5]) Margin = 45;
    else Margin = 1;
    QRect rect(Margin, Margin, width()-2*Margin,height()-2*Margin);
    if(!rect.isValid()) {
        return;
        if (debug) cout << "plot dimensions are invalid" << endl;
    }

    //clips all points outside plot area
    painter->setClipRect(rect.adjusted(+1,+1,-1,-1));

    //import plot data
    int numCurves;
    if (curveMap.count() == 0) numCurves =1;
    else numCurves = curveMap.count();

    if (data.T.length() ==0) numCurves =0;

    QList<double> yVect;
    QList<double> xVect;

    for (int z=0;z<numCurves;z++) {
        if (numCurves ==1) {
            yVect.append(data.Y.at(timeIndex));
            xVect.append(data.X.at(timeIndex));
        }
        else {
            yVect.clear();
            xVect.clear();
            yVect.append(curveMap.at(z).Y.at(timeIndex));
            xVect.append(curveMap.at(z).X.at(timeIndex));
        }
        QPolygonF polyline(yVect.count()); //stores points

        if (yVect.count() != xVect.count())
        {
            cout << "data mismatch. X&Y different lengths " <<
                    xVect.count()<<" " <<yVect.count()<<endl;
            return;
        }

        //first convert from units to pixels
        //store locations in polyline then draw points
        for (int j=0; j<yVect.count(); ++j) {
            double dx = xVect[j] - settings.minX;
            double dy = yVect[j] - settings.minY;
            double x = rect.left() + (dx* (rect.width()-1) / settings.spanX());
            double y = rect.bottom() - (dy* (rect.height()-1) / settings.spanY());

            polyline[j] = QPointF(x,y);
        }

        //draw curve lines
        if (linesShown) {
            if (numCurves ==1) linePen = QPen(colorForIds[lineColor]);
            else linePen = QPen(colorForIds[z%8+1]);
            linePen.setWidth(lineWidth);
            painter->setPen(linePen);
            painter->drawPolyline(polyline);
        }

        //draw points
        if (markersShown) {
            if (numCurves ==1) {
                painter->setPen(colorForIds[markerColor]);
                painter->setBrush(colorForIds[markerColor]);
            }
            else {
                painter->setPen(colorForIds[(z+2)%8+1]);
                painter->setBrush(colorForIds[(z+2)%8+1]);
            }
            for (int k=0;k<polyline.count();++k) {

                //tag by x interval (non-uniform sections will be tagged according to dx)
                if ((tagByDx)&&(k>0)&&(k+1<polyline.count())) {
                    double dx1 = polyline[k+1].x()-polyline[k].x();
                    double dx2 = polyline[k].x()-polyline[k-1].x();

                    double diff = fabs(dx1-dx2);

                    if (diff > 2*dx1 || diff > 2*dx2) {
                        if (dx1 > dx2) offset += 1;
                        else offset -= 1;

                        if (numCurves ==1) {
                            painter->setPen(colorForIds[(markerColor+offset)%8]);
                            painter->setBrush(colorForIds[(markerColor+offset)%8]);
                        }
                        else {
                            painter->setPen(colorForIds[(z+2+markerColor)%8+1]);
                            painter->setBrush(colorForIds[(z+2+markerColor)%8+1]);
                        }
                    }//end if diff
                }//end tagByDx

                switch (markerType)
                {
                case 0:
                    painter->drawEllipse(polyline[k],markerSize,markerSize);
                    break;
                case 1:
                    //draw square marker - need to offset by half length to center marker
                    painter->drawRect(polyline[k].x()-markerSize,polyline[k].y()-markerSize,
                                      2*markerSize,2*markerSize);
                    break;
                case 2:{
                        QPolygonF triangle(3);
                        triangle[0] = QPointF(polyline[k].x(), polyline[k].y()+markerSize);
                        triangle[1] = QPointF(polyline[k].x()+.86603*2*markerSize, polyline[k].y()-markerSize/2.);
                        triangle[2] = QPointF(polyline[k].x()-.86603*2*markerSize, polyline[k].y()-markerSize/2.);
                        painter->drawPolygon(triangle);}
                    break;
                default:
                    painter->drawEllipse(polyline[k],markerSize,markerSize);
                }//end switch

            }//end for loop
        }//end drawing markers
    }//end loop over curves
}
//draws labels
void PlotWidget::drawLabels(QPainter *painter)
{
    static const QColor colorForIds[8] = {Qt::black,Qt::red,Qt::green, Qt::yellow,
                                          Qt::blue,Qt::magenta,Qt::cyan,Qt::white };
    painter->setClipRect(0,0,width(),height());
    //Plot(data) title
    painter->setFont(QFont("Arial", 14));
    if (labelsVisible[0]) {
        painter->setPen(colorForIds[labelColors[0]]);
        if (labelsVisible[6] || labelsVisible[5]) painter->drawText(width()/2-70,35,name);
            else painter->drawText(width()/2-50,70,name);
    }

    painter->setFont(QFont("Arial", 10));
    //time index label
    if (labelsVisible[1]){
        painter->setPen(colorForIds[labelColors[1]]);
        painter->drawText(20,20,indexLabel);
    }
    //lower limit labels
    if (labelsVisible[2]){
        painter->setPen(colorForIds[labelColors[2]]);
        painter->drawText(10,height()-10,lowerLimitLabels);
    }
    //upper limit labels
    if (labelsVisible[2]) {
        painter->setPen(colorForIds[labelColors[2]]);
        painter->drawText(backButton->x()-85,20,upperLimitLabels);
    }
    //display mode label
    if (labelsVisible[3]) {
        painter->setPen(colorForIds[labelColors[3]]);
        if (this->displayMode == ZOOM) painter->drawText(width()-70,height()-10,displayModeLabel);
        else painter->drawText(width()-60,height()-10,displayModeLabel);
    }
    //time label
    if (labelsVisible[4]) {
        painter->setPen(colorForIds[labelColors[4]]);
        if (labelsVisible[6] || labelsVisible[5]) painter->drawText(70,70,timeLabel);
        else painter->drawText(50,50,timeLabel);
    }
}
/****** End Drawing functions ******/


/******fits plot boundries to data max/mins********/
//find maximum values in array
double PlotWidget::getMax(QList<double> vect)
{
    double maxX;
    maxX = vect.at(0);
    for (int i=0; i<vect.count();i++)
    {
        if (vect.at(i) > maxX) maxX = vect.at(i);
    }
    return maxX;
}
//find minimum values in array
double PlotWidget::getMin(QList<double> vect)
{
    double minX;
    minX = vect.at(0);
    for (int i=0; i<vect.count();i++)
    {
        if (vect.at(i) < minX) minX = vect.at(i);
    }
    return minX;
}
plotLimits PlotWidget::getBoundingLims(QList<double> x, QList<double> y)
{
    plotLimits p;
    p.xMax = getMax(x);
    p.yMax = getMax(y);
    p.xMin = getMin(x);
    p.yMin = getMin(y);

    return p;
}

void PlotWidget::setPlotBoundaries(plotLimits p, PlotSettings *settings)
{
    if (settings == 0) settings = &zoomStack[curZoom];
    settings->maxX = p.xMax;
    settings->minX = p.xMin;
    settings->maxY = p.yMax;
    settings->minY = p.yMin;
}
void PlotWidget::resetPlotLimits(plotLimits p)
{
    if (isActive || isApplyAllActive) { //so only active plot(s) respond to control dialog
        PlotSettings settings;
        PlotSettings *s = &settings;
        setPlotBoundaries(p,s);

        if  ((p.yMin >= p.yMax) || (p.xMin>=p.xMax) ){
            QMessageBox::information(this, tr("Plot Limits"),
                                     tr("Invalid Limits\n") );
        }
        else {
            zoomStack.resize(curZoom +1);
            zoomStack.append(settings);
            forward();
        } //end else
        emit needUpdateControlDialog("view"); //updates control dialog

    } // end if
}
void PlotWidget::optimizePlotBoundaries()
{
    if (isActive || isApplyAllActive) {
        const QList<double> &yVect = data.Y.at(timeIndex);
        const QList<double> &xVect = data.X.at(timeIndex);
        plotLimits p;
        p=getBoundingLims(xVect,yVect);

        if (curveMap.length() >0) {
            QList<double> xLims;
            QList<double> yLims;

            for (int i=0;i<curveMap.length();i++) {

                QList<double> x= curveMap.at(i).X.at(timeIndex);
                QList<double> y= curveMap.at(i).Y.at(timeIndex);

                xLims.append(getMax(x));
                xLims.append(getMin(x));
                yLims.append(getMax(y));
                yLims.append(getMin(y));
            }//end for
            p = getBoundingLims(xLims,yLims);
        }//end if

        resetPlotLimits(p);
    }
}
void PlotWidget::initializePlotBoundaries()
{
    //get data
    const QList<double> &yVect = data.Y.at(timeIndex);
    const QList<double> &xVect = data.X.at(timeIndex);

    //get limits
    plotLimits p;
    p= getBoundingLims(xVect,yVect);

    //for case of multiple datasets
    if (curveMap.length() >0) {
        QList<double> xLims;
        QList<double> yLims;

        for (int i=0;i<curveMap.length();i++) {

            QList<double> x= curveMap.at(i).X.at(timeIndex);
            QList<double> y= curveMap.at(i).Y.at(timeIndex);

            xLims.append(getMax(x));
            xLims.append(getMin(x));
            yLims.append(getMax(y));
            yLims.append(getMin(y));
        }//end for
        p = getBoundingLims(xLims,yLims);
    }//end if

    //put new limits into settings pointer
    setPlotBoundaries(p);

    //get settings pointer
    PlotSettings *settings = &zoomStack[curZoom];

    //provide viable view in case of zero vector
    if ((p.yMax - p.yMin) == 0) settings->maxY = settings->minY +1;

    if (!fullScale) settings->adjust(); //round limits
    updateMaxMinLabels();
    emit needUpdateControlDialog("view"); //updates control dialog
}
/******End fits plot boundries to data max/mins********/




