#ifndef TYPES_H
#define TYPES_H

#include <QString>
#include <QInputDialog>
#include <QList>
#include <QThread>
#include <QDir>
#include <iostream>
#include <cmath>

enum MODE {ZOOM =0, STEP=1, AUTO=2, FREEZE=3, XZOOM=4, YZOOM=5, XYZOOM=6, SELECTION=40, DESELECTION=-1};

extern double getDouble(double def,QString str);

struct plotLimits
{
    double xMin;
    double xMax;
    double yMin;
    double yMax;
};

struct FofX
{
    QList<double> X;
    QList<double> Y;
    double time;
};

class PlotSettings //helper class with control over plot boundaries
{
public:
    PlotSettings();//constructor
    void scroll(int dx,int dy); //moves view by specified amount
    void adjust();              // adjusts both axes to rounder, more human friendly numbers
    void info();                // gives info to std output on xy boundaries

    double spanX() const {return maxX-minX; } //x span in x units (not in pixel width)
    double spanY() const {return maxY-minY; } //y span in y units (not in pixel width)

    double minX; //minimum x value shown
    double maxX; //max x value shown
    int numXTicks; //number of tick marks on x-axis

    double minY; //minimum y value shown
    double maxY; //max y value shown
    int numYTicks; //number of tick marks on y-axis

private:
    static void adjustAxis(double &min, double &max, int &numTicks); //adjusts single axis tickmarks to more friendly numbers
    /*end PlotSettings*/
};

class xytVector
{
public:
    //contructors
    xytVector(){};
    xytVector(QString n);

    //member variables
    QString name;
    QList< QList<double> > Y;
    QList< QList<double> > X;
    QList<double> T;

    //set functions
    void setUniformX(QList<double> list, int timeSteps);

    //applicator functions
    void x_log_c_plus_x(double c);
    void x_times_y();
    void y_divide_x();
    void x_p(double p);
    void x_p_times_y(double p);
    void y_p(double p);
    void dy_dx();
};

class getSelectedWidgets : public QThread
{
    Q_OBJECT //insures getSelectedWidgets can receive signals
public:
    getSelectedWidgets();        //constructor
    getSelectedWidgets(int need);//constructor with initialized numNeeded
    virtual void run();//provides run routine that has its own event loop.

private:
    int numSelected; //current number of selected widgets
    int numNeeded;   //number of widgets needed-> when numSelected=numNeeded, thread returns.

public slots:
    void changeSelected(int n); //updates number of selected widgets
};

#endif // TYPES_H
