#ifndef XVSMAINWINDOW_H
#define XVSMAINWINDOW_H

#include <QMainWindow>
#include "controlWindow.h"
#include "plotWidget.h"
#include "ui_xvsmainwindow.h"
#include "types.h"

#include "include/sdf.h"
#include "include/bbhutil.h"
#include "include/xy.h"
#include "include/xy_types.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

//Main window class: holds plot widgets and contains layout and file i/o code
class xvsMainWindow : public QMainWindow, public Ui::xvsMainWindow
{
    Q_OBJECT
public:
    xvsMainWindow(QWidget *parent = 0); //constructor

private:
    //private variables
    double ifuzz; //comparison allowence
    bool allCheckBoxState; //Apply to all property
    bool ctrl_spaceKills; //shortcut on/off property
    bool controlButtonDepressed; //tracks control button status

    //initialization of menus and status bar
    void createMenuActions();
    void createPlotActions();
    void createMenus();
    void createContextActions();
    void createContextMenus();
    void createStatusBar();

    //status bar labels
    QLabel *statusLabel;
    QLabel *formulaLabel;

    //convert to PXYDVBIND
    PXYDVBIND get_PXYDVBIND();

    //Main Menus
    QMenu *fileMenu;
    QMenu *miscMenu;
    QMenu *exportMenu;
    QMenu *openMenu;

    //Main Window Actions- operations that are inserted into menus
    QAction *maximizeActiveWindowAction; //hides all other plots and maximizes selected plot
    QAction *killActiveWindow; //duh.

    //file menu actions
    QAction *exitHandlerAction; //determines whether exit should be accepted
    QAction *showControlDialogAction; //shows dialog at mouse position
    QAction *getHelpAction; //pops up help dialog
        //openMenu actions
        QAction *openTextFileAction; //opens text file and graphs in new plot
        QAction *openSdfFileAction; //opens sdf file

    //Misc Menu actions
    QAction *set_insert_fuzz; //sets insert fuzz (margin of acceptable error when comparing data)
    QAction *reset_insert_fuzz;//resets fuzz to 1e-10
    QAction *ctrl_space_kills_windows; //toggles this shortcut
        //exportMenu actions
        QAction *makeMpegAction; //exports animation of data w/ current settings to mpeg.
        QAction *exportImagesAction;  //exports single frame (or all frames if apply-all property active)
        QAction *exportToTextAction; //exports single or multiple datasets to text format.
        QAction *exportToSdfAction; //exports single dataset to sdf format.

    //contextMenu (right click options)
    QMenu *kill_window_menu;
    QAction *killSingleWindowAction;
    QAction *killAllWindowAction;

    QMenu *apply_control_menu;
    QAction *toggle_apply_all_on; //toggles ApplytoAll button & property

    QMenu *applyHistMenu;

    QMenu *apply1Menu;
    QList<QAction *> applyMiscTable;
    QAction *dy_dxAction;
    QAction *x_log_c_plus_xAction;
    QAction *x_log_one_plus_xAction;
    QAction *x_times_yAction;
    QAction *y_divide_xAction;
    QAction *x_pAction;
    QAction *x_p_times_yAction;
    QAction *y_pAction;

    QMenu *applyNMenu;
    QAction *mergeAction;
    QAction *splitAction;
    QAction *h_sq_convergence_testAction;
    QAction *h_lin_convergence_testAction;
    QAction *h_p_convergence_testAction;

    /*QAction *dy_dx_midAction;
    QAction *x_dy_dxAction;
    QAction *int_dxAction;
    QAction *rint_dxAction;
    QAction *int_dx_divide_xAction;
    QAction *int_x_p_y_dxAction;*/

    QMenu *compareMenu;
    QList<QAction *> compareTable;
    QAction *compare_full_exactAction;
    QAction *compare_full_fuzzAction;
    QAction *compare_x_exactAction;
    QAction *compare_x_fuzzAction;
    QAction *compare_y_exactAction;
    QAction *compare_y_fuzzAction;
    QAction *compare_t_exactAction;
    QAction *compare_t_fuzzAction;
    QAction *compare_xy_exactAction;
    QAction *compare_xy_fuzzAction;

    //plot Window related members
    PlotWidget *arrayPlotWidgets[25]; //array of plots, determines max # as well NOTE: w/ plotwidget, min size only allows 12 to be shown at one time
    int activeWidgetIndex; //tracks active widget wrt cursor
    int numPlotWidgets; //duh.
    int gridSize;//gridsize for windows (i.e gridsize of 2 holds 4 windows)
    bool isPlotMaximized; //property that tracks maximization of a chosen plot
    void changeActivePlot(int i); //changes active plot to plot i
    void cloneActivePlot(); //produces a clone of active plot in new plot window
    bool levelFormatted(QList<PlotWidget *> list);
    bool isMergable(QList<PlotWidget*> plots);

    //selection mode area
    QList<PlotWidget *> selectedWidgets; //keeps track of widgets selected in selection mode
    void activateSelectionMode(int num); //activates selection mode by using a getSelectedWidgets thread (see types.h)
    void deactivateSelectionMode(); //deactivates selection mode and clears the selectedWidgets array
    void (xvsMainWindow::*pt2Member)(); //provides pointer to function for use when getSelectedWidgets returns
    bool selectionMode; //tracks activity of selection mode. true=on.

    //Control Dialog
    ControlWindow *controlDialog; //control dialog with many plot options
    void setupControlDialog(); //initializes control dialog connections
    void setupPlotWindow(PlotWidget *w);//create plot connections to control window

    //exit handler functions
    bool okToQuit(); //pointless as of july20,2010
    bool okToClose(); //pointless as of july20,2010

protected:
    void closeEvent(QCloseEvent *event); //reimplemented to give "are you sure you want to exit" dialog
    void contextMenuEvent(QContextMenuEvent *event); //creates custom context menu
    void keyPressEvent(QKeyEvent *event);//implements all keyboard shortcuts
    void keyReleaseEvent(QKeyEvent *event); //deals with control key toggling
    void mousePressEvent(QMouseEvent *event);//deals with killing selection mode

signals:
    void closeWindow();//quit/exit xvs signal
    void allButtonToggled(bool);//toggles apply to all property
    void widgetChanged(QString);//signals a change in active window, used to update controlDialog
    void selectionModeActivated(int); //activates selection mode for plot widgets. arg changes plot mode.
    void selectedWidgetsChanged(int); //tracks # of selected widgets
    void endSelectionMode(int); //activated by middle mouse button click

private slots:
    //plotWidget slots
    void addPlotWidget();//bug free addition of plotWidget to mainWindow -> please don't touch
    void removePlotWidget();//removes active plot
    void organizePlotWidgets();//reshuffles plots into best grid
    void maximizeActivePlot(); //hides all other plots
    void killAllPlotWindows(); //duh.

    //update value slots
    void updateControlDialogValues(QString s);//resets values when active plot changes; string arg options: updateAll, view, name, index, mode; connected to needControlDialogUpdate signal in plotWidget
    void cursorPosUpdate(QPointF pos, int i); //updates cursor position label
    void setAllCheckBoxState(bool check); //keeps the "apply to all" property modifiers in sync

    //menu related slots
    void openTextFile(); //opens a text file in new window
    FofX readSdfFile(const char *filename, int tLevel); //uses exterior lib here (bbhutil)
    void openSdfFile();
    void exportToTextFile();
    void exportToSdfFile(); //uses exterior lib here (bbhutil)
    QList<int> getCurrentPlots(); //function that returns relevant plots (all, selected, or active in that order)

    void callPlottingRoutine(); //not implemented

    //Help menu slot
    void getHelp(); //help dialog pops up

    //Misc menu slots
    void getControlDialog(); //opens control dialog
    void getInsertFuzz();    //user inputs 'insert fuzz in dialog
    void ResetInsertFuzz();  //resets ifuzz to 1e-10
    void toggle_ctrl_space_kills_windows(bool cskw); //shortcut toggled on/off

    //Exit menu slot
    void exitHandeler();

    //APPLICATOR FUNCTIONS

    //apply xy menu
    void x_log_c_plus_x();
    void x_log_one_plus_x();
    void x_times_y();
    void x_p();
    void x_p_times_y();
    void y_p();
    void y_divide_x();
    void dy_dx();

    //convergence test menu
    QList<xytVector> getDiffs(QList<PlotWidget *> wgs, double p);
    void h_sq_convergence_test();
    void h_lin_convergence_test();
    void h_p_convergence_test();

    //compare menu
    bool compare_x(PlotWidget *w1, PlotWidget *w2, double fuzz);
    bool compare_y(PlotWidget *w1, PlotWidget *w2, double fuzz);
    bool compare_t(PlotWidget *w1, PlotWidget *w2, double fuzz);
    void compare_full_exact();
    void compare_full_fuzz();
    void compare_x_exact();
    void compare_x_fuzz();
    void compare_y_exact();
    void compare_y_fuzz();
    void compare_t_exact();
    void compare_t_fuzz();
    void compare_xy_exact();
    void compare_xy_fuzz();

    //merge & split
    void merge();
    void split();

    //selection mode slots
    void selectFunction();
    void addSelectedWidget(int i);
    void removeSelectedWidget(int i);
};

#endif // XVSMAINWINDOW_H
